#!/usr/bin/ruby
# 
# make_quartus_vwf.rb
#
# vwf file generator
#
# Copyright Jun Makino All rights reserved.
# J. Makino, 2004/10/7-
$LOAD_PATH.unshift(File.dirname($0))
$LOAD_PATH.push(ENV["ACSROOT"]+"/src/utilities") if ENV["ACSROOT"] != nil
require "clop.rb"

options_definition_string = <<-END

  Description: Waveform generator for Quartus II
  Long description:
    Waveform generator for Quarus II
    (c) 2010- Jun Makino

    Takes the input like this
  
    clock cp_clk  20
    input bar0   
    input we
    input be     8
    input addr   8
    input datain 64
    1 1 0f  60 20
    1 1 0f  64 1111111111111111
    1 1 0f  60 21
    1 1 0f  64 2222222222222222
    1 1 0f  60 22
    1 1 0f  64 3333333333333333

    and create the vwf file. It assumes to take two inputs, 
    template vwf file and actual waveform file (mqv format, above).
    Maximum help is provided by the command

    "ruby make_quartus_vwf.rb --help".

    This program assumes that the target circuit is a simple,
    fully-synchronous circuit with a single global clock.  The
    name and frequency of the clock signal is specified by the
    line 

        clock name_of_the_clok_input_pin half_of_the_clock_period

    Here, name_of_the_clok_input_pin is the signal name for the
    clock, and half_of_the_clock_period is the half of the clock
    period in ns.

    Input singnals other than the clock is specified by

        input name_of_the_input_pin [number_of_bits]

    For example, 

       input bar0   

    defines one bit input signal with name bar0 , and

       input datain 64

    defines a 64-bit input with name datain

    Finally, actual input singnals should be given for every
   clock pulse, in one line with hexadecimal numbers separated by
   space(s).  The number of lines for the singnals determines the
   cycles for which the simulation is run (simulation is run for
   additional 20 clock cycles).

   The names and width of input signals must be the same as those
   appear in the template vwf file. This one is normally created by 
   using the waveform editor of Quartus II.


  Short name: 		-i
  Long name:		--input
  Value type:		string
  Default value:        none
  Variable name: 	infile
  Description:		Input mqv file name (required)
  Long description:
    This option specifies the file name for input mqv file.
    This is a necessary parameter. The file name must end with
    ".mqv"


  Short name: 		-t
  Long name:		--template
  Value type:		string
  Variable name: 	templateinfile
  Default value:        none
  Description:		Template Input vwf file name
  Long description:
    This option specifies the file name for input VSM file. 
    This is a necessary parameter.

  END

$clop = parse_command_line(options_definition_string, false)

input=    open($clop.infile,"r") 
templateinput=    open($clop.templateinfile,"r") 
outname = File.basename($clop.infile,".mqv")+".vwf"
print "output file name=", outname, "\n"
out = open(outname,"w+")

# copy the first part of the template
while (s = templateinput.gets) !~ /TRANSITION_LIST/
  out.print s
end

# read-in and parse input mqv file
inputs = []
input_bits = []
signals = []
while (s = input.gets)
  a = s.chomp.split;
  if a[0] == "clock" 
    clock_name = a[1]
    clock_cycle = a[2].to_i
  elsif a[0] == "input"
    inputs.push(a[1]);
    input_bits.push( (a.length >= 3)?  a[2].to_i : 1)
  else
    signals.push(a)
  end
end
p inputs
p input_bits
p signals

#generate clock
s = <<-END
TRANSITION_LIST("#{clock_name}")
{
        NODE
        {
                REPEAT = #{signals.length+20};
                LEVEL 0 FOR #{clock_cycle};
                LEVEL 1 FOR #{clock_cycle};
        }
}

END
out.print s

#generate other signals
inputs.each_with_index{|x,i|
  nsignals = input_bits[i]
  nsignals.times{|loc|
    name = x
    if nsignals > 1
      name += "["+loc.to_s+"]"
    end
   s= <<-END
TRANSITION_LIST("#{name}")
{
        NODE
        {
                REPEAT = 1;
                LEVEL 0 FOR #{clock_cycle*2};
END
    out.print s
    signals.each{|sig|
      val = sig[i].to_i(16)[loc]
      out.print "                LEVEL #{val} FOR #{clock_cycle*2};\n"
    }
    out.print "        }\n"
    out.print "}\n\n"
    }
}
while (s = templateinput.gets) !~ /DISPLAY_LINE/
# just skip lines
end
out.print s
while (s = templateinput.gets)
  out.print s
end

